<?php

if (!defined('ABSPATH')) exit;

class gdpos_core_admin extends d4p_admin_core {
    public $plugin = 'gd-power-search-for-bbpress';

    public $task = false;

    public function __construct() {
        $this->url = GDPOS_URL;

        add_action('gdpos_plugin_init', array($this, 'core'));

        add_filter('plugin_action_links', array($this, 'plugin_actions'), 10, 2);
        add_filter('plugin_row_meta', array($this, 'plugin_links'), 10, 2);
    }

    public function plugin_actions($links, $file) {
        if ($file == $this->plugin.'/'.$this->plugin.'.php') {
            $settings_link = '<a href="options-general.php?page=gd-power-search-for-bbpress">'.__("Settings", "gd-power-search-for-bbpress").'</a>';
            array_unshift($links, $settings_link);
        }

        return $links;
    }

    function plugin_links($links, $file) {
        if ($file == $this->plugin.'/'.$this->plugin.'.php') {
            $links[] = '<a target="_blank" href="https://support.dev4press.com/kb/product/'.$this->plugin.'/">'.__("Knowledge Base", "gd-power-search-for-bbpress").'</a>';
            $links[] = '<a target="_blank" href="https://support.dev4press.com/forums/forum/plugins/'.$this->plugin.'/">'.__("Support Forum", "gd-power-search-for-bbpress").'</a>';
        }

        return $links;
    }

    public function current_url($with_panel = true, $with_task = true) {
        $page = 'options-general.php?page='.$this->plugin;

        if ($with_panel && $this->panel !== false && $this->panel != '') {
            $page.= '&panel='.$this->panel;
        }

        if ($with_task && isset($this->task) && $this->task !== false && $this->task != '') {
            $page.= '&task='.$this->task;
        }

        return self_admin_url($page);
    }

    public function title() {
        return __("GD Power Search for bbPress", "gd-power-search-for-bbpress");
    }

    public function core() {
        parent::core();

        add_action('admin_menu', array($this, 'admin_menu'));

        if (gdpos_settings()->is_install()) {
            add_action('admin_notices', array($this, 'install_notice'));
        }

        if (gdpos_settings()->is_update()) {
            add_action('admin_notices', array($this, 'update_notice'));
        }
    }

    public function install_notice() {
        if (current_user_can('install_plugins') && $this->page === false) {
            echo '<div class="updated"><p>';
            echo __("GD Power Search Pro for bbPress is activated and it needs to finish installation.", "gd-power-search-for-bbpress");
            echo ' <a href="options-general.php?page=gd-power-search-for-bbpress">'.__("Click Here", "gd-power-search-for-bbpress").'</a>.';
            echo '</p></div>';
        }
    }

    public function update_notice() {
        if (current_user_can('install_plugins') && $this->page === false) {
            echo '<div class="updated"><p>';
            echo __("GD Power Search Pro for bbPress is updated, and you need to review the update process.", "gd-power-search-for-bbpress");
            echo ' <a href="options-general.php?page=gd-power-search-for-bbpress">'.__("Click Here", "gd-power-search-for-bbpress").'</a>.';
            echo '</p></div>';
        }
    }

    public function enqueue_scripts($hook) {
        if ($this->page !== false) {
            d4p_admin_enqueue_defaults();

            wp_enqueue_style('fontawesome', GDPOS_URL.'d4plib/resources/fontawesome/css/font-awesome.min.css');

            wp_enqueue_style('d4plib-font', $this->file('css', 'font', true), array(), D4P_VERSION.'.'.D4P_BUILD);
            wp_enqueue_style('d4plib-shared', $this->file('css', 'shared', true), array(), D4P_VERSION.'.'.D4P_BUILD);
            wp_enqueue_style('d4plib-admin', $this->file('css', 'admin', true), array('d4plib-shared'), D4P_VERSION.'.'.D4P_BUILD);
            wp_enqueue_style('gdpos-admin', $this->file('css', 'admin-core'), array('d4plib-admin'), gdpos_settings()->file_version());

            wp_enqueue_script('d4plib-admin', $this->file('js', 'admin', true), array('jquery', 'wp-color-picker'), D4P_VERSION.'.'.D4P_BUILD, true);
            wp_enqueue_script('gdpos-admin', $this->file('js', 'admin-core'), array('d4plib-admin'), gdpos_settings()->file_version(), true);

            if ($this->panel == 'about') {
                wp_enqueue_style('d4plib-grid', $this->file('css', 'grid', true), array(), D4P_VERSION.'.'.D4P_BUILD);
            }
        }
    }

    public function admin_init() {
        do_action('gdpos_admin_init');
    }

    public function admin_menu() {
        $this->page_ids[] = add_options_page($this->title(), __("GD Power Search", "gd-power-search-for-bbpress"), 'activate_plugins', $this->plugin, array($this, 'plugin_interface'));

        $this->admin_load_hooks();
    }

    public function admin_load_hooks() {
        foreach ($this->page_ids as $id) {
            add_action('load-'.$id, array($this, 'load_admin_page'));
        }
    }

    public function current_screen($screen) {
        if ($screen->id == 'settings_page_gd-power-search-for-bbpress') {
            $this->page = true;
        }

        if ($this->page) {
            if (isset($_GET['panel']) && $_GET['panel'] != '') {
                $this->panel = d4p_sanitize_slug($_GET['panel']);
            } else {
                $this->panel = 'settings';
            }

            if (isset($_GET['task']) && $_GET['task'] != '') {
                $this->task = d4p_sanitize_slug($_GET['task']);
            }
        }

        if (isset($_POST['gdpos_handler']) && $_POST['gdpos_handler'] == 'postback') {
            require_once(GDPOS_PATH.'core/admin/postback.php');

            new gdpos_admin_postback();
        } else if (isset($_GET['gdpos_handler']) && $_GET['gdpos_handler'] == 'getback') {
            require_once(GDPOS_PATH.'core/admin/getback.php');

            new gdpos_admin_getback();
        }
    }

    public function load_admin_page() {
        $this->help_tab_sidebar();

        do_action('gdpos_load_admin_page_'.$this->page);

        if ($this->panel !== false && $this->panel != '') {
            do_action('gdpos_load_admin_page_'.$this->page.'_'.$this->panel);
        }

        $this->help_tab_getting_help();
    }

    public function install_or_update() {
        $install = gdpos_settings()->is_install();
        $update = gdpos_settings()->is_update();

        if ($install) {
            include(GDPOS_PATH.'forms/install.php');
        } else if ($update) {
            include(GDPOS_PATH.'forms/update.php');
        }

        return $install || $update;
    }

    public function plugin_interface() {
        if (!$this->install_or_update()) {
            require_once(GDPOS_PATH.'forms/front.php');
        }
    }
}

global $_gdpos_core_admin;
$_gdpos_core_admin = new gdpos_core_admin();

function gdpos_admin() {
    global $_gdpos_core_admin;
    return $_gdpos_core_admin;
}
